--
-- The finite state machine in control of the transmitter
--
LIBRARY ieee;
USE  IEEE.STD_LOGIC_1164.all;
USE  IEEE.STD_LOGIC_ARITH.all;
USE  IEEE.STD_LOGIC_UNSIGNED.all;

			
ENTITY transmitter_control_unit IS
      Port (	
  	    CLK 		:	IN	STD_LOGIC;
		CD   		:	IN	STD_LOGIC;
		Enable_TX 	:	IN	STD_LOGIC;
		ST 			:	IN	STD_LOGIC;
		TC8   		:	IN	STD_LOGIC;
		F   		:	OUT	STD_LOGIC_VECTOR(1 DOWNTO 0);
		S 			:	OUT	STD_LOGIC_VECTOR(1 DOWNTO 0);
		LDD   		:	OUT	STD_LOGIC;
		CE   		:	OUT	STD_LOGIC;
		ET   		:	OUT	STD_LOGIC
	);

END transmitter_control_unit;


ARCHITECTURE FSM_like OF transmitter_control_unit IS


	TYPE State_type IS (Idle, Load_Data_Reg, Load_Shift_Reg, Send_Start_bit, Send_D0, Send_D1_to_D7, Send_EPB, Send_Stop_bit, Wait_not_ready) ;  

	ATTRIBUTE syn_encoding : STRING;
	ATTRIBUTE syn_encoding OF State_type : TYPE IS "sequential";

	SIGNAL present_state, future_state	: State_type ; 


CONSTANT Reset		: State_type := Idle; -- The fisrt state. 

BEGIN


state_register: PROCESS (CD,CLK)
	BEGIN
						
	IF CD = '1' THEN 			-- reset counter
		present_state <= Reset; 
	ELSIF (CLK'EVENT and CLK = '1') THEN	-- Synchronous register (D-type flip-flop)
		present_state <= future_state; 
	END IF;

END PROCESS state_register;

------------------------- CC1: Combinational system for calculating next state

CC_1: PROCESS (present_state, TC8, Enable_TX, ST)
	BEGIN
		CASE present_state IS
			WHEN Idle => 
			    if (ST = '1' and Enable_TX = '1') THEN
					future_state <= Load_Data_Reg;
			    else
			    	future_state <= Idle ;
				 end if;   
			 
			WHEN Load_Data_Reg => 
				future_state <= Load_Shift_Reg;
				
			WHEN Load_Shift_Reg => 
				future_state <= Send_Start_bit ;
				
			WHEN Send_Start_bit => 
				future_state <= Send_D0 ;
				
			WHEN Send_D0 => 
				future_state <= Send_D1_to_D7 ;
				
			WHEN Send_D1_to_D7 => 
			    if (TC8 = '0') THEN
			        future_state <= Send_D1_to_D7;
			    else
				     future_state <= Send_EPB ;
				 end if;
				 
			WHEN Send_EPB => 
				future_state <= Send_Stop_bit ;
				
			WHEN Send_Stop_bit => 
				future_state <= Wait_not_ready ;
				
			WHEN Wait_not_ready => 
			    if (ST = '1') THEN
			        future_state <= Wait_not_ready;
			    else
			        future_state <= Idle ;
             end if;

			END CASE ;		

END PROCESS CC_1;

------------- CS_2: combinational system for calculating outputs at every state (Moore machine)
CC_2: PROCESS (present_state)
	BEGIN

		CASE present_state IS
				WHEN Idle => 
					LDD <= '0';
					F 	<= "00";
					S 	<= "00";
					CE 	<= '0';
					ET 	<= '0';     
	
				WHEN Load_Data_Reg => 
					LDD <= '1';
					F 	<= "00";      -- Channe1 Ch0 --> Marking at level '1'
					S 	<= "00";
					CE 	<= '0';
					ET 	<= '0';
	
				WHEN Load_Shift_Reg => 
					LDD <= '0';
					F 	<= "00";
					S 	<= "11";	-- Load the data register 
					CE 	<= '0';
					ET 	<= '0';     
					
				WHEN Send_Start_bit => 
					LDD <= '0';
					F 	<= "01";     -- Channe1 Ch1
					S 	<= "00";
					CE 	<= '0';
					ET 	<= '0';
					
				WHEN Send_D0 => 
					LDD <= '0';
					F 	<= "10";     -- Channe1 Ch2 is the RSO
					S 	<= "01";     -- and prepare to shift data on the next rising edge 
					CE 	<= '1';		-- allow the counter start counting
					ET 	<= '0';
					
				WHEN Send_D1_to_D7 => 
					LDD <= '0';
					F 	<= "10";
					S 	<= "01";     -- shift data while the counter is not full
					CE 	<= '1';
					ET 	<= '0';
					
				WHEN Send_EPB => 
					LDD <= '0';
					F 	<= "11";   	-- Ch3 to send the Even parity Bit 
					S 	<= "00";
					CE 	<= '0';
					ET 	<= '0';
					
				WHEN Send_Stop_bit => 
					LDD <= '0';
					F 	<= "00";   -- Ch0 to send the stop bit and marking at '1' 
					S 	<= "00";
					CE 	<= '0';
					ET 	<= '1';   -- at the same state generate a pulse of "end of transmission" 
					
				WHEN Wait_not_ready =>  
					LDD <= '0';
					F 	<= "00";
					S 	<= "00";
					CE 	<= '0';
					ET 	<= '0';
		END CASE ;		


END PROCESS CC_2;

-- Place other logic if necessary

END FSM_like;